﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Threading;

namespace Josso.Agent.Lib
{
    /// <summary>
    /// the user from the josso server
    /// </summary>
    public class ZjsUser
    {
        /// <summary>
        /// The domain name of the user is in this property
        /// </summary>
        internal const string JOSSO_USER_DN = "josso.user.dn";

        /// <summary>
        /// The timeout of the cache data in minutes.
        /// </summary>
        private const int DEFAULT_CACHE_TIMEOUT = 300;

        /// <summary>
        /// The period that each clean is trigerred in minutes.
        /// </summary>
        private const int CLEAN_PERIOD = 30;

        /// <summary>
        /// The Cache to avoid to connect to josso to much time.
        /// </summary>
        private static readonly Dictionary<string, CacheData> _cache;

        /// <summary>
        /// The cache timeout.
        /// </summary>
        private static int? _cacheTimeout;

        /// <summary>
        /// Instantiate the cache and start the cleanning thread.
        /// </summary>
        static ZjsUser()
        {
            _cache = new Dictionary<string, CacheData>();
            new Timer(cleanData, null, new TimeSpan(0, CLEAN_PERIOD, 0),
                      new TimeSpan(0, CLEAN_PERIOD, 0));
        }

        /// <summary>
        /// the name of the user
        /// </summary>
        public string Name { get; private set; }

        /// <summary>
        /// properties of the user
        /// </summary>
        public Dictionary<string, string> Properties { get; private set; }

        /// <summary>
        /// The session id of the user
        /// </summary>
        internal string SessionId { get; private set; }

        /// <summary>
        /// Josso Cache timeout in seconds. Try to read if from appsettings in "JossoCacheTimeout".
        /// </summary>
        private static int CacheTimeout
        {
            get
            {
                if (!_cacheTimeout.HasValue)
                {
                    int configTimeout;
                    if (int.TryParse(ConfigurationManager.AppSettings["JossoCacheTimeout"], out configTimeout))
                    {
                        _cacheTimeout = configTimeout;
                    }
                    else
                    {
                        _cacheTimeout = DEFAULT_CACHE_TIMEOUT;
                    }
                }

                return _cacheTimeout.Value;
            }
        }

        /// <summary>
        /// instanciate the user
        /// </summary>
        /// <param name="josso">The josso proxy</param>
        /// <param name="sessionId">the Id of the session</param>
        /// <returns>the user object</returns>
        internal static ZjsUser UserFactory(ZjsJosso josso, string sessionId)
        {
            lock (_cache)
            {
                CacheData data;
                if (_cache.TryGetValue(sessionId, out data)
                    && DateTime.Now - data.DateTime < new TimeSpan(0, 0, CacheTimeout))
                {
                    return data.User;
                }

                string userName;
                Dictionary<string, string> properties;
                try
                {
                    userName = josso.GetUserName(sessionId);
                    properties = josso.GetProperties(sessionId);
                }
                catch (System.Exception)
                {
                    return null;
                }

                if (string.IsNullOrEmpty(userName))
                    return null;

                ZjsUser user = new ZjsUser
                {
                    Name = userName,
                    Properties = properties,
                    SessionId = sessionId,
                };

                Console.WriteLine("Caching session {0} of user {1}, timeout in {2} seconds.",
                                  sessionId, userName, CacheTimeout);
                _cache[sessionId] = new CacheData(user);
                return user;
            }
        }

        /// <summary>
        /// Clear the cache for the user reprensented by it session id.
        /// </summary>
        /// <param name="sessionId"></param>
        internal static void Clear(string sessionId)
        {
            lock (_cache)
            {
                if (_cache.ContainsKey(sessionId))
                {
                    _cache.Remove(sessionId);
                    Console.WriteLine("Clear cache session {0}", sessionId);
                }
            }
        }

        /// <summary>
        /// The method that clean the cache in a separate thread.
        /// </summary>
        /// <param name="arg"></param>
        private static void cleanData(object arg)
        {
            lock (_cache)
            {
                List<string> keys = new List<string>(_cache.Keys);
                foreach (string session in keys)
                {
                    CacheData data = _cache[session];
                    if (new TimeSpan(0, CLEAN_PERIOD, 0) < DateTime.Now - data.DateTime)
                    {
                        Console.WriteLine("The cache info {0} has exprired, cleanning up", session);
                        _cache.Remove(session);
                    }
                }
            }
        }

        /// <summary>
        /// The object that is cached, with the date when it was cached
        /// </summary>
        private class CacheData
        {
            public CacheData(ZjsUser user)
            {
                User = user;
                DateTime = DateTime.Now;
            }

            public ZjsUser User { get; private set; }

            public DateTime DateTime { get; private set; }
        }
    }
}